@@ -129,9 +129,6 @@ VTK_MYSQL_TYPENAME_MACRO(float, MYSQL_TYPE_FLOAT);
 VTK_MYSQL_TYPENAME_MACRO(double, MYSQL_TYPE_DOUBLE);
 VTK_MYSQL_TYPENAME_MACRO(long long, MYSQL_TYPE_LONGLONG);
 VTK_MYSQL_TYPENAME_MACRO(unsigned long long, MYSQL_TYPE_LONGLONG);
-VTK_MYSQL_TYPENAME_MACRO(const char*, MYSQL_TYPE_STRING);
-VTK_MYSQL_TYPENAME_MACRO(char*, MYSQL_TYPE_STRING);
-VTK_MYSQL_TYPENAME_MACRO(void*, MYSQL_TYPE_BLOB);
 
 template <typename T>
 bool vtkMySQLIsTypeUnsigned(T)
