@@ -168,7 +168,7 @@
 
 //----------------------------------------------------------------------------
 // clang-format off
-// this unicode code to keysym table is meant to provide keysym similar to XLookupString,
+// this unicode code to keysym table is meant to provide keysym similar to the X Window System's XLookupString(),
 // for Basic Latin and Latin1 unicode blocks.
 // Generated from xlib/X11/keysymdef.h
 // Duplicated in Rendering/UI/vtkWin32RenderWindowInteractor.cxx
@@ -195,16 +195,18 @@ static const char* UnicodeToKeySymTable[256] = {
 };
 
 //----------------------------------------------------------------------------
-// This table is meant to provide keysym similar to XLookupString from macOS VKeys (Events.h)
-// that are not mapped in the unicode table above.
-static const char* MacKeyCodeToKeySymTable[128] = { 
-  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, 
-  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, 
-  nullptr, nullptr, nullptr, nullptr, "Return",nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, 
-  "Tab", nullptr, nullptr, "BackSpace", nullptr, "Escape", nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, 
-  nullptr, "period", nullptr, "asterisk", nullptr, "plus", nullptr, "Clear", nullptr, nullptr, nullptr, "slash", "KP_Enter", nullptr, "minus", nullptr, 
-  nullptr, nullptr, "KP_0", "KP_1", "KP_2", "KP_3", "KP_4", "KP_5", "KP_6", "KP_7", nullptr, "KP_8", "KP_9", nullptr, nullptr, nullptr, 
-  "F5", "F6", "F7", "F3", "F8", nullptr, nullptr, nullptr, nullptr, "Snapshot", nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, 
+// This table is meant to provide keysym similar to X Window System's XLookupString()
+// from macOS virtual keys that are not mapped in the unicode table above.
+// See the kVK_* enums in HIToolbox/Events.h. For example, kVK_Return = 0x24, and so
+// the string "Return" appears at index 36 in this list.
+static const char* MacKeyCodeToKeySymTable[128] = {
+  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
+  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
+  nullptr, nullptr, nullptr, nullptr, "Return",nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
+  "Tab", nullptr, nullptr, "BackSpace", nullptr, "Escape", nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
+  nullptr, "period", nullptr, "asterisk", nullptr, "plus", nullptr, "Clear", nullptr, nullptr, nullptr, "slash", "KP_Enter", nullptr, "minus", nullptr,
+  nullptr, nullptr, "KP_0", "KP_1", "KP_2", "KP_3", "KP_4", "KP_5", "KP_6", "KP_7", nullptr, "KP_8", "KP_9", nullptr, nullptr, nullptr,
+  "F5", "F6", "F7", "F3", "F8", "F9", nullptr, "F11", nullptr, "Snapshot", nullptr, nullptr, nullptr, "F10", nullptr, "F12",
   nullptr, nullptr, "Help", "Home", "Prior", "Delete", "F4", "End", "F2", "Next", "F1", "Left", "Right", "Down", "Up", nullptr };
 // clang-format on
 
