@@ -8,18 +8,20 @@ readonly name="fides"
 readonly ownership="Fides Upstream <kwrobot@kitware.com>"
 readonly subtree="ThirdParty/$name/vtk$name"
 readonly repo="https://gitlab.kitware.com/third-party/fides.git"
-readonly tag="for/vtk-20210827-master"
+readonly tag="for/vtk-20250507-master-2e44017f"
 readonly paths="
 .gitattributes
-CMakeLists.txt
-fides
-cmake/FidesModule.cmake
-cmake/FidesExportHeaderTemplate.h.in
-cmake/fides_generate_export_header.cmake
-thirdparty
 LICENSE.txt
 README.md
 README.kitware.md
+
+CMakeLists.txt
+cmake/FidesModule.cmake
+cmake/FidesExportHeaderTemplate.h.in
+cmake/fides_generate_export_header.cmake
+
+fides/
+thirdparty/
 "
 
 extract_source () {
